#! /usr/bin/python

## gui.py
## Copyright (C) 2007 Ivan Uemlianin
## Contact: ivan@llaisdy.com
## 
## This file is part of Trefnydd.
##
## Trefnydd is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## Trefnydd is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Trefnydd (i.e., doc/gpl.txt); if not, write to the Free Software
## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
## or see http://www.gnu.org/licenses/gpl.txt


"""
========
trefnydd
========

Package:   trefnydd
Module:    gui
Copyright: 2007 Ivan A. Uemlianin
Contact:   ivan@llaisdy.com
Licence:   GNU GPL (see doc/gpl.txt)

gui
===

"""


import os

import wx
import wx.grid
import wx.html

class NewCorpusDialog(wx.Dialog):
    def __init__(self, parent):
        self.name = ''
        self.type = 'Speech'
        # main frame
        wx.Dialog.__init__(self, parent, -1, "Choose Corpus Properties")

        sizer = wx.BoxSizer(wx.VERTICAL) # Main vertical sizer

        # name entry
        name_l = wx.StaticText(self, -1, "Name:")
        #name_t = wx.TextCtrl(self)
        self.name = wx.TextCtrl(self)
        # TODO: for validation see wxPython in action p284f

        # type entry
        type_l = wx.StaticText(self, -1, "Type:")
        # TODO: do I need this pad? how not?
        pad = wx.StaticText(self, -1, "")
        type_r1 = wx.RadioButton(self, -1, "Speech")
        type_r2 = wx.RadioButton(self, -1, "Text")
        for r in [type_r1, type_r2]:
            self.Bind(wx.EVT_RADIOBUTTON, self.OnRadio, r)

        # Layout with sizers
        fgs = wx.FlexGridSizer(3, 2, 5, 5)
        fgs.Add(name_l, 0, wx.ALIGN_RIGHT)
        #fgs.Add(name_t, 0, wx.EXPAND)
        fgs.Add(self.name, 0, wx.EXPAND)
        fgs.Add(type_l, 0, wx.ALIGN_RIGHT)
        fgs.Add(type_r1, 0, wx.EXPAND)
        fgs.Add(pad, 0, wx.ALIGN_RIGHT)
        fgs.Add(type_r2, 0, wx.EXPAND)
        fgs.AddGrowableCol(1)
        sizer.Add(fgs, 0, wx.EXPAND|wx.ALL, 5)

        # Use standard button IDs
        okay = wx.Button(self, wx.ID_OK)
        okay.SetDefault()
        cancel = wx.Button(self, wx.ID_CANCEL)

        btns = wx.StdDialogButtonSizer()
        btns.AddButton(okay)
        btns.AddButton(cancel)
        btns.Realize()
        sizer.Add(btns, 0, wx.EXPAND|wx.ALL, 5)

        self.SetSizer(sizer)
        sizer.Fit(self)

    def OnRadio(self, evt):
        self.type = evt.GetEventObject().GetLabel()


class NewModelDialog(wx.Dialog):
    def __init__(self, parent):
        self.name = ''
        self.type = 'Acoustic Model (AM)'
        # main frame
        wx.Dialog.__init__(self, parent, -1, "Choose Model Properties")

        sizer = wx.BoxSizer(wx.VERTICAL) # Main vertical sizer

        # name entry
        name_l = wx.StaticText(self, -1, "Name:")
        #name_t = wx.TextCtrl(self)
        self.name = wx.TextCtrl(self, -1, 'htk')
        # TODO: for validation see wxPython in action p284f

        # type entry
        type_l = wx.StaticText(self, -1, "Type:")
        # TODO: do I need this pad? how not?
        pad = wx.StaticText(self, -1, "")
        type_r1 = wx.RadioButton(self, -1, "Acoustic Model (AM)")
        type_r2 = wx.RadioButton(self, -1, "Language Model (LM)")
        for r in [type_r1, type_r2]:
            self.Bind(wx.EVT_RADIOBUTTON, self.OnRadio, r)

        # Layout with sizers
        fgs = wx.FlexGridSizer(3, 2, 5, 5)
        fgs.Add(name_l, 0, wx.ALIGN_RIGHT)
        #fgs.Add(name_t, 0, wx.EXPAND)
        fgs.Add(self.name, 0, wx.EXPAND)
        fgs.Add(type_l, 0, wx.ALIGN_RIGHT)
        fgs.Add(type_r1, 0, wx.EXPAND)
        fgs.Add(pad, 0, wx.ALIGN_RIGHT)
        fgs.Add(type_r2, 0, wx.EXPAND)
        fgs.AddGrowableCol(1)
        sizer.Add(fgs, 0, wx.EXPAND|wx.ALL, 5)

        # Use standard button IDs
        okay = wx.Button(self, wx.ID_OK)
        okay.SetDefault()
        cancel = wx.Button(self, wx.ID_CANCEL)

        btns = wx.StdDialogButtonSizer()
        btns.AddButton(okay)
        btns.AddButton(cancel)
        btns.Realize()
        sizer.Add(btns, 0, wx.EXPAND|wx.ALL, 5)

        self.SetSizer(sizer)
        sizer.Fit(self)

    def OnRadio(self, evt):
        self.type = evt.GetEventObject().GetLabel()


class AboutBox(wx.Dialog):
    """Cribbed from wxPython In Action Listing 6.10
    """
    text = '''<html>
    <body>
    <h1>Trefnydd</h1>
    <p><b>Trefnydd</b>: A Speech Recognition Toolkit.</p>
    <p>Copyright &copy; 2006 Ivan Uemlianin.</p>
    </body>
    </html>
    '''

    def __init__(self, parent):
        wx.Dialog.__init__(self, parent, -1, 'About Trefnydd',
                          size=(440, 400) )

        html = wx.html.HtmlWindow(self)
        html.SetPage(self.text)
        button = wx.Button(self, wx.ID_OK, "OK")

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(html, 1, wx.EXPAND|wx.ALL, 5)
        sizer.Add(button, 0, wx.ALIGN_CENTER|wx.ALL, 5)

        self.SetSizer(sizer)
        self.Layout()

        
class DataDisplay(wx.Panel):
    st_col_labs = ['min', 'max', 'label']
    def __init__(self, parent, debug=False):
        wx.Panel.__init__(self, parent, style=wx.SUNKEN_BORDER)
        self.parent = parent
        self.debug = debug          
        self.SetBackgroundColour("green")
        self.data = None
        self.dataParent = None
        self.name = 'View/Edit'
        self.head = wx.StaticText(self, -1, self.name)
        self.head.SetBackgroundColour("yellow") # todo: not working
        self.body = None # see draw()
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        #self.sizer.Add(self.head, 0, wx.EXPAND|wx.ALL)
        self.sizer.Add(self.head, flag=wx.EXPAND)
        self.SetSizer(self.sizer)
        self.sizer.Fit(self)
        self.Bind(wx.grid.EVT_GRID_CELL_CHANGE, self.onChange)
                    
    def draw(self):
        # todo: size/resize grid to fit window
        rows = len(self.data)
        cols = len(self.st_col_labs)
        self.body = wx.grid.Grid(self)
        self.body.CreateGrid(rows, cols) 
        for i in range(cols):
            self.body.SetColLabelValue(i, self.st_col_labs[i])
        for row in range(rows):
            thisRowData = self.data[row].toList()
            for col in range(cols):
                self.body.SetCellValue(row, col, thisRowData[col])
        
    def report(self):
        self.body = wx.html.HtmlWindow(self)
        if "gtk2" in wx.PlatformInfo:
            self.body.SetStandardFonts()
        self.body.SetPage(self.data)

    def display(self, item, itemParent, mode='grid'):
        self.data = item
        self.dataParent = itemParent
        self.name = itemParent.name
        self.head.SetLabel(self.name)
       
        if mode == 'grid':
            self.draw()
        elif mode == 'report':
            self.report()
    
        self.sizer.Add(self.body, 1, flag=wx.EXPAND) #|wx.ALL)
        # what does the extra wx.ALL bit do?
        self.sizer.Fit(self)

    def onChange(self, evt):
        # todo: this assumes self.data is a SegmentationTier
        # from a SpeechCorpus
        # design a more generic solution
        # (get it working first!)
        row = evt.GetRow()
        col = evt.GetCol()
        setattr(self.data[row], self.st_col_labs[col], 
                self.body.GetCellValue(row, col))
        self.data.setChanged()
        self.dataParent.setChanged()
        print '%s(%s, %s).changed = %s'% (self.dataParent.name,
                                          row, col, self.dataParent.changed)
        
        
    
