#! /usr/bin/python

## project.py
## Copyright (C) 2007 Ivan Uemlianin
## Contact: ivan@llaisdy.com
## 
## This file is part of Trefnydd.
##
## Trefnydd is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## Trefnydd is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Trefnydd (i.e., doc/gpl.txt); if not, write to the Free Software
## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
## or see http://www.gnu.org/licenses/gpl.txt


"""
========
trefnydd
========

Package:   trefnydd
Module:    project
Copyright: 2007 Ivan A. Uemlianin
Contact:   ivan@llaisdy.com
Licence:   GNU GPL (see doc/gpl.txt)

project
=======

"""


import os

from trefnydd.utils import file_IO, location

class TrefnyddProject(location.Located):
    def __init__(self, dirn='', debug=False):
        location.Located.__init__(self)
        self.debug = debug
        self.name = ''
        self.corpora = []
        self.pd = None
        self.models = []
        self.engines = []
        if dirn:
            self.getProject(dirn)        

    def setPD(self, prd):
        self.pd = prd
        pdDir = file_IO.getDir(os.path.join(self.homeDir, self.name, 'pd'))
        self.pd.homeDir = pdDir
        if self.debug:
            print 'TrefnyddProject.setPD: pdDir = %s' % pdDir

    def getProject(self, dirn):
        # nb: dirn might be dir or full path
        if not os.path.exists(dirn):
            self.createProject(dirn)
        self.homeDir, self.name = os.path.split(os.path.abspath(dirn))
        # todo: load any corpora, etc
        if os.path.exists(os.path.join(self.homeDir, self.name, 'corpora')):
            self.loadCorpora()
        if os.path.exists(os.path.join(self.homeDir, self.name, 'pd')):
            self.loadPD()
        if os.path.exists(os.path.join(self.homeDir, self.name, 'models')):
            self.loadModels()
        if os.path.exists(os.path.join(self.homeDir, self.name, 'engines')):
            self.loadEngines()
        
    def createProject(self, dirn):
        os.mkdir(dirn)

    def save(self): # todo
        if self.debug is True:
            dirn = file_IO.getDir(os.path.join(self.homeDir, self.name))
            print 'project.save: dirn = %s' % dirn
        if self.corpora:
            if self.debug is True:
                print 'Saving corpora ...'
            self.saveObjListInDir(self.corpora, 'corpora')
        if self.pd:
            if self.debug is True:
                print 'Saving pd ...'
            self.saveObjListInDir([self.pd], 'pd')
        if self.models:
            self.saveObjListInDir(self.models, 'models')
        if self.engines:
            self.saveObjListInDir(self.engines, 'engines')

    def saveObjListInDir(self, objList, dirn):
        thisDir = file_IO.getDir(os.path.join(self.homeDir, self.name, dirn))
        for obj in objList:
            obj.setHomeDir(thisDir)
            obj.save()

    def loadCorpora(self):
        if self.debug is True:
            print 'TrefnyddProject.loadCorpora'
        from trefnydd.lib import corpus
        corpusDir = os.path.join(self.homeDir, self.name, 'corpora')
        for corpusName in os.listdir(corpusDir):
            # todo: DANGER: speech or text corpus!?
            c = corpus.SpeechCorpus(self.debug)
            corpusHomeDir = os.path.abspath(os.path.join(corpusDir,
                                                         corpusName))
            c.load(corpusHomeDir)
            self.addCorpus(c)

    def loadPD(self): # todo
        if self.debug is True:
            print 'TrefnyddProject.loadPD()'
        from trefnydd.lib import pd
        self.pd = pd.PronunciationDictionary(self.debug)
        self.pd.homeDir = os.path.join(self.homeDir, self.name, 'pd')
        self.pd.load()
    
    def loadModels(self):
        pass
    
    def loadEngines(self):
        pass

    def loadCorpusIntoModelBuilder(self, corp, mb):
        for utt in corp:
            if self.debug: print 'utt: %s' % utt.name
            mb.corpus.add_utterance(utt)
    
    def addCorpus(self, corpus):
        self.corpora.append(corpus)
        corpusDir = file_IO.getDir(os.path.join(self.homeDir, self.name,
                                                'corpora', corpus.name))
        corpus.setHomeDir(corpusDir)

    def addModel(self, model):
        self.models.append(model)
        modelDir = file_IO.getDir(os.path.join(self.homeDir, self.name,
                                                'models', model.name))
        model.setHomeDir(modelDir)


def main():
    p = TrefnyddProject('/home/ivan/trefnydd/trefnydd/NonGUITestProject')
    # new corpus
    # corpus: import data
    # save project
    p.save()
    # load project
    
    
if __name__ == '__main__':
    main()
