#! /usr/bin/python

## tester.py
## Copyright (C) 2007 Ivan Uemlianin
## Contact: ivan@llaisdy.com
## 
## This file is part of Trefnydd.
##
## Trefnydd is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## Trefnydd is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Trefnydd (i.e., doc/gpl.txt); if not, write to the Free Software
## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
## or see http://www.gnu.org/licenses/gpl.txt


"""
========
trefnydd
========

Package:   trefnydd
Module:    tester
Copyright: 2007 Ivan A. Uemlianin
Contact:   ivan@llaisdy.com
Licence:   GNU GPL (see doc/gpl.txt)

tester
======

"""


import os, sys
trefnydd_root = os.path.dirname(os.path.dirname(os.getcwd()))
if trefnydd_root not in sys.path:
    sys.path.append(trefnydd_root)

# from trefnydd.gui import gui

def main(debug, projName=''):
    import trefnydd.trefnydd_gui
    trefnydd.trefnydd_gui.main(projName, debug)

def no_gui(debug, projName): # p.save() working with pd
    from trefnydd.project import TrefnyddProject
    from trefnydd.lib.corpus import SpeechCorpus
    from trefnydd.lib.pd import PronunciationDictionary
    from trefnydd.lib import g2p, phones
    p = TrefnyddProject(projName, debug)
    p.addCorpus(getCorpus('amser'))

    # build acoustic model with HTKAMBuilder: working 
    from trefnydd.lib.ambuilder import ambuilder
    hb = ambuilder.HTKAMBuilder(True)
    p.addModel(hb)
    p.loadCorpusIntoModelBuilder(p.corpora[0], p.models[0])
    hb.labelType = 'Phone'
    hb.setup()
    hb.extractFeatures()
    hb.defineModelParameters()
    open('log.html', 'w').write(hb.pprintlog('html'))

    # todo:
    # logs to html via docutils: could be better but OK
    # do all above via gui


##    # load project, pd: working
##    print p.pd.phoneSet

##     # new pd; save project: working
##     pd = PronunciationDictionary()
##     p.setPD(pd)
##     pd.importFromFile('Grapheme Set', '../meta/cymraeg.graphemeset')
##     pd.importFromFile('Phone Set', '../meta/ipa.phoneset')
##     pd.importFromFile('G2P Rules', '../meta/cymraeg.g2p')
##     print 'Imported meta/*.'
##     p.save()
##     print 'Saved.'

##     # new speech corpus; save project
##     c = SpeechCorpus()
##     c.name = 'llj'
##     c.importData('/home/ivan/trefnydd/trefnydd/llj_tiny')
##     p.addCorpus(c)
##     p.save()
##     p.setHomeDir('/home/ivan/trefnydd/trefnydd/NewHomeDir')
##     p.save()

def getCorpus(name='amser'):
    from trefnydd.lib.corpus import SpeechCorpus    
    c = SpeechCorpus()
    c.homeDir = name
    c.importData() # phone labels
    #c.importData('lll_tiny') # word labels
    return c
    

if __name__ == '__main__':
    debug = True
    projName = 'TestTrefnyddProject'
    args = ['-n'] # sys.argv[1:]
    if '-n' in args:
        #projName = 'NonGUI%s' % projName
        no_gui(debug, projName)
    else:
        main(debug, projName)
